--============================================================================= 
--
-- Simple PWM sequence
--
--============================================================================= 

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity pwm_seq is
  generic (
    cnt_clk_max : integer := 20000;
    cnt_pulse_max : integer := 1250
  );
  port (
    clk : in std_logic;
    sel : in std_logic;
    led_out : out std_logic_vector(7 downto 0)
  );
end entity;

architecture behaviour of pwm_seq is
  signal pwm_out : std_logic;

  signal polarity : std_logic;
  signal polarity_nxt : std_logic;

  signal duty_cycle : integer range 0 to cnt_pulse_max;
  signal duty_cycle_nxt : integer range 0 to cnt_pulse_max;

  signal cnt_pulse : integer range 0 to cnt_pulse_max;
  signal cnt_pulse_nxt : integer range 0 to cnt_pulse_max;

  signal cnt_clk : integer range 0 to cnt_clk_max;
  signal cnt_clk_nxt : integer range 0 to cnt_clk_max;

  signal cnt_end_impuls : std_logic;
  signal cnt_end_impuls_nxt : std_logic;

begin
  -------------------------------------------------------------------------------
  -- Register
  -------------------------------------------------------------------------------
  proc_name : process (clk, sel)
  begin
    if sel = '0' then
      polarity <= '0';
      duty_cycle <= 0;
      cnt_pulse <= 0;
      cnt_clk <= 0;
      cnt_end_impuls <= '0';
    elsif rising_edge(clk) then
      polarity <= polarity_nxt;
      duty_cycle <= duty_cycle_nxt;
      cnt_pulse <= cnt_pulse_nxt;
      cnt_clk <= cnt_clk_nxt;
      cnt_end_impuls <= cnt_end_impuls_nxt;
    end if;
  end process proc_name;
  -------------------------------------------------------------------------------
  -- Duty Cycle
  -------------------------------------------------------------------------------
  process (duty_cycle, cnt_end_impuls, polarity)
  begin
    duty_cycle_nxt <= duty_cycle;
    polarity_nxt <= polarity;
    if cnt_end_impuls = '1' then
      if (polarity = '0') then
        if duty_cycle < (cnt_pulse_max - 1) then
          duty_cycle_nxt <= duty_cycle + 1;
          polarity_nxt <= '0';
        else
          polarity_nxt <= '1';
        end if;
      elsif (polarity = '1') then
        if (duty_cycle > 1) then
          duty_cycle_nxt <= duty_cycle - 1;
          polarity_nxt <= '1';
        else
          polarity_nxt <= '0';
        end if;
      end if;
    end if;
  end process;

  -------------------------------------------------------------------------------
  -- Counting
  -------------------------------------------------------------------------------
  process (cnt_clk, cnt_pulse, cnt_end_impuls)
  begin
    cnt_clk_nxt <= cnt_clk;
    cnt_pulse_nxt <= cnt_pulse;
    cnt_end_impuls_nxt <= cnt_end_impuls;
    if cnt_clk = (cnt_clk_max - 1) then
      cnt_clk_nxt <= 0;
      cnt_end_impuls_nxt <= '1';
    else
      cnt_clk_nxt <= cnt_clk + 1;
      cnt_end_impuls_nxt <= '0';
    end if;

    if cnt_pulse < (cnt_pulse_max - 1) then
      cnt_pulse_nxt <= cnt_pulse + 1;
    else
      cnt_pulse_nxt <= 0;
    end if;
  end process;
  -------------------------------------------------------------------------------
  -- Pulsing
  -------------------------------------------------------------------------------
  process (clk)
  begin
    if rising_edge(clk) then
      if (duty_cycle > cnt_pulse) then
        pwm_out <= '1';
      else
        pwm_out <= '0';
      end if;
    end if;
  end process;

  -------------------------------------------------------------------------------
  -- LED output
  -------------------------------------------------------------------------------
  led_out(7 downto 4) <= (others => pwm_out) when sel = '1' else
  (others => '0');
  led_out(3 downto 0) <= (others => not pwm_out) when sel = '1' else
  (others => '0');

end architecture;